/// <reference types="node" />
import { EventEmitter } from "events";
import { HTTPApi } from "./api";
import { GuardMode } from "./types";
import { HubResponse } from "./models";
import { ParameterArray } from "./interfaces";
import { P2PInterface } from "./../p2p/interfaces";
import { WatermarkSetting } from "../p2p/types";
import { Device } from "./device";
export declare class Station extends EventEmitter implements P2PInterface {
    private api;
    private hub;
    private log;
    private dsk_key;
    private dsk_expiration;
    private p2p_session;
    private parameters;
    private currentDelay;
    private reconnectTimeout?;
    static readonly CHANNEL: number;
    constructor(api: HTTPApi, hub: HubResponse);
    getStateID(state: string, level?: number): string;
    getStateChannel(): string;
    update(hub: HubResponse): void;
    isStation(): boolean;
    isDeviceStation(): boolean;
    getDeviceType(): number;
    getHardwareVersion(): string;
    getMACAddress(): string;
    getModel(): string;
    getName(): string;
    getSerial(): string;
    getSoftwareVersion(): string;
    getIPAddress(): string;
    private loadParameters;
    getParameter(param_type: number): string;
    private getDSKKeys;
    isConnected(): boolean;
    close(): void;
    connect(): Promise<void>;
    private onFinishDownload;
    private onStartDownload;
    private onStopLivestream;
    private onStartLivestream;
    setGuardMode(mode: GuardMode): Promise<void>;
    getCameraInfo(): Promise<void>;
    getStorageInfo(): Promise<void>;
    private onAlarmMode;
    private onCameraInfo;
    private onCommandResponse;
    private onConnect;
    private onDisconnect;
    getParameters(): ParameterArray;
    private getCurrentDelay;
    private resetCurrentDelay;
    private scheduleReconnect;
    rebootHUB(): Promise<void>;
    setStatusLed(device: Device, value: boolean): Promise<void>;
    setAutoNightVision(device: Device, value: boolean): Promise<void>;
    setAntiTheftDetection(device: Device, value: boolean): Promise<void>;
    setWatermark(device: Device, value: WatermarkSetting): Promise<void>;
    enableDevice(device: Device, value: boolean): Promise<void>;
    startDownload(path: string, private_key: string): Promise<void>;
    cancelDownload(device: Device): Promise<void>;
    startLivestream(device: Device): Promise<void>;
    stopLivestream(device: Device): Promise<void>;
    isLiveStreaming(device: Device): boolean;
}
